class Player:
    def __init__(self, id, name, identifiers, ping):
        self.id = id
        self.name = name
        self.identifiers = Identifiers(identifiers)
        self.ping = ping

    @staticmethod
    def parse(data: dict):
        return Player(data.get('id'), data.get('name'), data.get('identifiers'), data.get('ping'))


class Identifiers:
    def __init__(self, identifiers):
        self.raw = identifiers

        self.license = None
        self.steam = None
        self.xbl = None
        self.discord = None
        self.live = None

        if identifiers is not None:
            for identifier in identifiers:
                service, id = identifier.split(':')
                if service == "license":
                    self.license = id
                elif service == "steam":
                    self.steam = id
                elif service == "xbl":
                    self.xbl = id
                elif service == "discord":
                    self.discord = id
                elif service == "live":
                    self.live = id



from fivem.player import Player


class Server:
    def __init__(self, hostname, server, players: list[Player], max_players, resources: list[str], vars: list[str], game_type, map_name):
        self.hostname = hostname
        self.server = server
        self.players = players
        self.max_players = max_players
        self.resources = resources
        self.vars = vars
        self.game = Game(game_type, map_name)

    @staticmethod
    def parse(dynamic: dict, info: dict, players: list[Player]):
        return Server(dynamic.get('hostname'), info.get('server'), players, dynamic.get('sv_maxclients'), info.get('resources'), info.get('vars'), dynamic.get('gametype'), dynamic.get('mapname'))


class Game:
    def __init__(self, game_type, map_name):
        self.game_type = game_type
        self.map_name = map_name



import json
import aiohttp

from fivem.player import Player
from fivem.server import Server


class FiveM:
    """Base class for a FiveM server"""

    def __init__(self, host, port):
        self.host = host
        self.port = port

    async def get_server(self) -> Server:
        dynamic = await self.get_dynamic_raw()

        info = await self.get_info_raw()
        players = list[Player]()
        if (dynamic['clients'] > 0):
            players = await self.get_players()

        return Server.parse(dynamic, info, players)

    async def get_players(self) -> list[Player]:
        players = list()
        for rawPlayer in await self.get_players_raw():
            players.append(Player.parse(rawPlayer))
        return players

    async def get_dynamic_raw(self) -> dict:
        return await self.make_request('dynamic.json')

    async def get_info_raw(self) -> dict:
        return await self.make_request('info.json')

    async def get_players_raw(self) -> list[dict]:
        return await self.make_request('players.json')

    async def make_request(self, uri):
        base = 'http://{}:{}/{}'
        url = base.format(self.host, self.port, uri)

        async with aiohttp.ClientSession() as session:
            try:
                async with session.get(url) as response:
                    data = await response.text()
                    return json.loads(data)
            except aiohttp.ClientConnectorError as e:
                raise FiveMServerOfflineError


class FiveMServerOfflineError(Exception):
    pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fivem-api",
    use_scm_version=True,
    author="Sander Jochems",
    author_email="contact@sanderjochems.com",
    description="A library to query info, players and resources from FiveM server",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/Sander0542/fivem-api",
    packages=setuptools.find_packages(exclude=['tests', 'tests.*']),
    install_requires=[
        'aiohttp>=3.8.1'
    ],
    setup_requires=[
        'setuptools_scm'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
